library(tidyverse)

# Set working directory to location of files
setwd("path/to/downloaded/files")

# Use read_delim to read in the results data file.
# The file in this example is a table of 16011 data points
# recording 8 variables, corresponding to 121 current interruption cycles.
df <- read_delim("chien_fastgitt_opxrd.txt")

# Define a function ici_example which performs the calculations required
ici_example <- function(df, tmin = 1, tmax = 10, V = 0.63056, A = 1.5E4) {

  # A new data frame 'sd' is created which adds the columns "state" (charge,
  # discharge, or 'R' (rest)), a unique counter variable 'rest', and step time.
  # It also modifies the charge variable so that the first Q value at the start
  # of each cycle is 0.
  sd <- df %>%
    # Now assign unique "state" (sign of current) and rest counter
    mutate(state = ifelse(I > 0, "charge", ifelse(I < 0, "discharge", "R")), 
           rest = imap(state, ~ ifelse(.x != "R" & state[.y-1] == "R", 1, 0)) %>%
             as.numeric %>%
             replace_na(0) %>%
             cumsum + 1
    ) %>%
    # Assign step time
    group_by(rest, state) %>%
    mutate(step.t = t - t[1]) %>%
    ungroup %>%
    # Reset charge to zero at start of each cycle.
    group_by(cyc.n) %>%
    mutate(Q = Q - Q[1]) %>%
    ungroup

  # If last charge-rest loop is not complete, it is removed, to avoid errors
  if (last(sd$state != "R")) {
    sd <- sd %>% filter(rest < max(rest))
  }

  # Collect data point for each individual rest
  # from each charge-rest loop, immediately before the rest. These values will
  # be the "t", "E", "Q" etc values recorded in the processed data.
  summary <- sd %>%
    group_by(rest) %>%
    filter(state != "R") %>%
    summarise_all(last)

  # For each rest cycle, a linear regression of E(we) vs t^1/2 is performed.
  # The intercept 'E0' and the gradient 's' are recorded, together with their
  # standard deviations
  regression <- sd %>%
    group_by(rest) %>%
    filter(state == "R", step.t >= tmin, step.t <= tmax) %>%
    group_modify(~ lm(E_p ~ sqrt(step.t), data = .x) %>%
                   with(
                     data.frame(
                       E0 = coef(summary(.))[1, 1],                
                       E0_err = coef(summary(.))[1, 2], #and error
                       s = coef(summary(.))[2, 1], #and gradient
                       s_err = coef(summary(.))[2, 2] #and error
                     )))

  # The output data frame is produced by combining the summary and regression
  # data frames, and the quantities R, k, dE/dt and D are calculated.
  out <- Reduce(function(x, y) merge(x, y, by = "rest"), 
                list(summary, regression)) %>%
   mutate(R = (E - E0) / (I / 1000), R_err = E0_err / (abs(I) / 1000),
           k = -1 * s / (I / 1000), k_err = s_err / (abs(I) / 1000),
           dEdt = (E-lag(E, 2))/(t-lag(t, 2)),
          D = (4/pi) * ((V/A) * (dEdt/s))^2
    )

  # The output data frame is returned.
  return(out)

}

# The processed data is called by using the function ici_example on the
# data frame df with the default arguments.
processed <- ici_example(df)

# ggplot for E vs t
ggplot(processed, aes(x = t, y = E)) +
  geom_path(aes(color = state), size = 1) +
  theme_bw() +
  scale_color_brewer(palette = "Set1") +
  labs(x = "t / s", y = "E / V")

ggsave("E-t.png", width = 7, height = 3.5, units = "in", dpi = 200)

# ggplot for R vs E
ggplot(processed, aes(x = E, y = R * 1.327)) +
  geom_path(aes(color = state), size = 1) +
  theme_bw() +
  scale_color_brewer(palette = "Set1") +
  labs(x = "E / V", y = "R /"~Omega~"cm"^"2")

ggsave("R-E.png", width = 7, height = 3.5, units = "in", dpi = 200)

# ggplot for k vs E
ggplot(processed, aes(x = E, y = k * 1.327)) +
  geom_path(aes(color = state), size = 1) +
  theme_bw() +
  scale_color_brewer(palette = "Set1") +
  labs(x = "E / V", y = "k /"~Omega~"cm"^2~"s"^"-1/2")

ggsave("k-E.png", width = 7, height = 3.5, units = "in", dpi = 200)

# ggplot for D (log scale) vs E
ggplot(processed, aes(x = E, y = D)) +
  geom_path(aes(color = state), size = 1) +
  theme_bw() +
  scale_color_brewer(palette = "Set1") +
  scale_y_log10() +
  labs(x = "E / V", y = "D / cm"^"2"~"s"^"-1")

ggsave("D-E.png", width = 7, height = 3.5, units = "in", dpi = 200)
